######################################################################
##
## Copyright (C) 2008,  Simon Kagstrom
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Bookmarks in functions/files
##
## Licensed under the terms of GNU General Public License version 2 or
## later. See COPYING file distributed with Dissy for full text of the
## license.
##
######################################################################

class Bookmark:
    def __init__(self, comment, address):
        self.comment = comment
        self.location = location

class BookmarkSet:
    def __init__(self, filename):
        self.bookmarks = []
        self.md5sum = 0


